"""

Berk - a package for producing and managing MeerKAT continuum survey data.

"""

#from . import catalogs 
from . import startup

#__all__ = ['startUp']

from . import _version
__version__ = _version.get_versions()['version']
